% Parameters used for mirror matrix computations %%%%%%%%%%%%%%%%%%%%%%%%%%
global rmir npt thr
rmir = 0.6;         % mirror radius in meters
npt = 151;          % number of points for numerical integration on a grid npt x npt
thr = 0;            % threshold to kill small matrix elements (safer to stay at 0!)

% Modes parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global mm M
mm = list_even_modes(8);    % use only even modes for mismatch
M = length(mm);

% Arm cavity parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

itf.RCi = 1416;  % input mirror ROC
itf.RCe = 1646;  % end mirror ROC
itf.L   = 3000;  % length

itf.ti = sqrt(0.0069);                  % input mirror  amplitude transmission 
itf.ri = sqrt(0.9931);                  % input mirror amplitude reflection
itf.li = 0;                             % input mirror losses

itf.te = sqrt(75e-6);                   % end mirror amplitude transmission
itf.le = 0;                             % end mirror losses
itf.re = sqrt(1 - itf.le - itf.te^2);   % end mirror amplitude reflection

% compute the arm cavity mode for latter computations
cav = cavity_mode(itf.RCi, itf.RCe, itf.L);

% compute finesse
itf.Finesse = pi * sqrt(itf.ri*itf.re) / (1 - itf.ri * itf.re);

% Recycling cavities parameteres %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

itf.tp = sqrt(0.04);                    % power recycling mirror amplitude transmission
itf.lp = 0;                             % power recycling mirror losses
itf.rp = sqrt(1 - itf.lp - itf.tp^2);   % power recycling mirror amplitude reflection

itf.ts = 1;                     % signal recycling mirror amplitude transmission
itf.ls = 0;                     % signal recycling mirror losses
itf.rs = 0;                     % signal recycling mirror amplitude reflection

itf.Lprc = 35.857;              % power recycling cavity length
itf.Lsrc = 11.0;                % signal recycling cavity length
itf.Schnupp = 0.0;              % Schnupp asymmetry

itf.fmod = 6270659;
itf.m = 0.3;

% compute the optimal curvature of recycling mirrors
Lp = itf.Lprc + cav.L1;
Ls = itf.Lsrc + cav.L1;
itf.RCp = Lp + cav.z0^2 / Lp;
itf.RCs = Ls + cav.z0^2 / Ls;
% compute beam size on recycling mirrors
itf.wp = cav.w0 * sqrt(1 + (Lp/cav.z0)^2);
itf.ws = cav.w0 * sqrt(1 + (Ls/cav.z0)^2);

% compute Gouy phase in recycling cavity propagation
%itf.phig_p = atan(Lp/cav.z0) - atan(cav.L1/cav.z0);
itf.phig_s = atan(Ls/cav.z0) - atan(cav.L1/cav.z0);

% to impose Non Degenerate Recycling cavities, set manually Gouy phase
itf.phig_p = 19.3 / 180 * pi;

% Length tunings %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
itf.phi_src = 0;                % signal recycling cavity tuning in radian
itf.DARM = 0;                   % microscopic tunings in meters of the main
itf.CARM = 0;                   % degrees of freedom
itf.PRCL = 0;
itf.MICH = 0;

% Aberrations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
itf.aRCin = itf.RCi;   % radii of curvature
itf.aRCen = itf.RCe;
itf.aRCiw = itf.RCi;   % radii of curvature
itf.aRCew = itf.RCe;
itf.aRCp = itf.RCp;
itf.aRCs = itf.RCs;

itf.fNI = 100e10;      % focal length of input mirror thermal lenses
itf.fWI = 100e10;

% end of parameters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Build arm cavities (with aberrations if defined)
itf.ncav = build_cavity(itf.RCi, itf.RCe, itf.L, itf.aRCin, itf.aRCen, itf.ri, itf.re, itf.li, itf.le, itf.fNI, mm);
itf.wcav = build_cavity(itf.RCi, itf.RCe, itf.L, itf.aRCiw, itf.aRCew, itf.ri, itf.re, itf.li, itf.le, itf.fWI, mm);

% build recycling mirrors (with aberrations if defined)
itf.Rp = itf.rp * mirror_reflection(itf.aRCp, itf.RCp, itf.wp, mm, rmir, npt, thr);
itf.Rs = 0;
itf.Tp = itf.tp * eye(length(mm));
itf.Ts = eye(length(mm));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% compute all thermal lenses for different powers
tic;
NPT = 300;
pabs = linspace(1e-6, 0.3, NPT);
f = 2700 ./ pabs;
L = lenses(f, itf.ncav.w1, list_even_modes(8), rmir, npt, thr);
el = toc;
disp(['Elapsed time: ' num2str(el) ' s'])     

% rescale lenses to maintain 00 resonant
for i=1:NPT
    LL(:,:,i) = squeeze(L(:,:,i)) / exp(1j * angle(L(1,1,i)));
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NO LOCKING PROCEDURE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% compute fields
tic;
Crc0 = zeros(M,M,NPT);
Csc0 = zeros(M,M,NPT);
Cdf0 = zeros(M,M,NPT);
Cn0 = zeros(M,M,NPT);
Cw0  = zeros(M,M,NPT);

Urc0 = zeros(M,M,NPT);
Usc0 = zeros(M,M,NPT);
Udf0 = zeros(M,M,NPT);
Un0  = zeros(M,M,NPT);
Uw0  = zeros(M,M,NPT);

Lrc0 = zeros(M,M,NPT);
Lsc0 = zeros(M,M,NPT);
Ldf0 = zeros(M,M,NPT);
Ln0  = zeros(M,M,NPT);
Lw0  = zeros(M,M,NPT);

itf.DARM = 0;
itf.CARM = 0;
itf.MICH = 0;
itf.PRCL = 0;

% loop over all lenses
h = waitbar(0, 'Computing static fields...');
for i=1:NPT
    waitbar(i/NPT, h, ['Computing static fields (', num2str(floor(100*i/NPT)), '%)']);

    % update ITF input thermal lenses
    itf.ncav.Lens = squeeze(LL(:,:,i));
    itf.wcav.Lens = squeeze(LL(:,:,i));

    % compute fields
    [Crc0(:,:,i), Csc0(:,:,i), Cdf0(:,:,i), Cn0(:,:,i), Cw0(:,:,i)] = dualrecycled_fields(itf, 0);
    [Urc0(:,:,i), Usc0(:,:,i), Udf0(:,:,i), Un0(:,:,i), Uw0(:,:,i)] = dualrecycled_fields(itf, +itf.fmod);
    [Lrc0(:,:,i), Lsc0(:,:,i), Ldf0(:,:,i), Ln0(:,:,i), Lw0(:,:,i)] = dualrecycled_fields(itf, -itf.fmod);
end
close(h);

% compute total DC fields
dccar0 = dc(squeeze(Crc0(:,1,:)), 1);
dcusb0 = dc(squeeze(Urc0(:,1,:)), 1);
dclsb0 = dc(squeeze(Lrc0(:,1,:)), 1);

% compute demodulated signals
[p0,q0] = ac(squeeze(Crc0(:,1,:)), squeeze(Urc0(:,1,:)), squeeze(Lrc0(:,1,:)), itf.m, 0);

el = toc;
disp(['Elapsed time: ' num2str(el) ' s'])     

% some plots
semilogy(pabs, abs(besselj(0,itf.m) * squeeze(Crc0(1,1,:))).^2)
hold all
semilogy(pabs, abs(besselj(1,itf.m) * squeeze(Urc0(1,1,:))).^2)
semilogy(pabs, abs(besselj(1,itf.m) * squeeze(Lrc0(1,1,:))).^2)
xlabel('Input mirror absorbed power [W]')
ylabel('PRC powers [W]')
grid
legend('CAR','USB','LSB')

figure()
plot(pabs, 2*(abs(squeeze(Cn0(1,1,:))) ./ abs(squeeze(Crc0(1,1,:)))).^2)
xlabel('Input mirror absorbed power [W]')
ylabel('FP recycling gain (00 mode)')
grid

figure()
semilogy(pabs, abs(squeeze(Crc0(1,1,:)).^2))
hold all
semilogy(pabs, abs(squeeze(Urc0(1,1,:)).^2))
semilogy(pabs, abs(squeeze(Lrc0(1,1,:)).^2))
xlabel('Input mirror absorbed power [W]')
ylabel('PRC recycling gains')
grid
legend('CAR','USB','LSB')

figure()
plot(pabs, abs(squeeze(Crc0(1,1,:)).^2), 'r--')
hold all
plot(pabs, abs(squeeze(Urc0(1,1,:)).^2), 'g--')
plot(pabs, abs(squeeze(Lrc0(1,1,:)).^2), 'b--')
plot(pabs, dccar0, 'r')
plot(pabs, dcusb0, 'g')
plot(pabs, dclsb0, 'b')
xlabel('Input mirror absorbed power [W]', 'fontsize', 15)
ylabel('PRC recycling gains', 'fontsize', 15)
grid
legend('CAR','USB','LSB')
title('Carrier TEM00 always resonant inside PRC and arms', 'fontsize', 15)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LOCKING PRCL AND CARM
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% compute fields
tic;
Crc = zeros(M,M,NPT);
Csc = zeros(M,M,NPT);
Cdf = zeros(M,M,NPT);
Cn  = zeros(M,M,NPT);
Cw  = zeros(M,M,NPT);

Urc = zeros(M,M,NPT);
Usc = zeros(M,M,NPT);
Udf = zeros(M,M,NPT);
Un  = zeros(M,M,NPT);
Uw  = zeros(M,M,NPT);

Lrc = zeros(M,M,NPT);
Lsc = zeros(M,M,NPT);
Ldf = zeros(M,M,NPT);
Ln  = zeros(M,M,NPT);
Lw  = zeros(M,M,NPT);

dccar = zeros(1,NPT);
dcusb = zeros(1,NPT);
dclsb = zeros(1,NPT);

Z = zeros(2, NPT);

% loop over all lenses
h = waitbar(0, 'Computing static fields...');
for i=1:NPT
    waitbar(i/NPT, h, ['Computing static fields (', num2str(floor(100*i/NPT)), '%)']);

    % update ITF input thermal lenses
    itf.ncav.Lens = squeeze(LL(:,:,i));
    itf.wcav.Lens = squeeze(LL(:,:,i));
    
    % use last locking point as a starting point
    if i>1
        Z(:,i) = Z(:,i-1);
    end
    
    % maximize sb in PRC
    Z(1,i) = fminsearch(@(x) -maximize_sb(itf, x, Z(2,i)), Z(1,i));
    % then maximize car in arm
    Z(2,i) = fminsearch(@(x) -maximize_car(itf, Z(1,i), x), Z(2,i));
    
    itf.PRCL = 1e-12 * Z(1,i);
    itf.CARM = 1e-12 * Z(2,i);

    % compute fields
    [Crc(:,:,i), Csc(:,:,i), Cdf(:,:,i), Cn(:,:,i), Cw(:,:,i)] = dualrecycled_fields(itf, 0);
    [Urc(:,:,i), Usc(:,:,i), Udf(:,:,i), Un(:,:,i), Uw(:,:,i)] = dualrecycled_fields(itf, +itf.fmod);
    [Lrc(:,:,i), Lsc(:,:,i), Ldf(:,:,i), Ln(:,:,i), Lw(:,:,i)] = dualrecycled_fields(itf, -itf.fmod);
    
    % compute total DC power
    dccar(i) = dc(squeeze(Crc(:,1,i)), 1);
    dcusb(i) = dc(squeeze(Urc(:,1,i)), 1);
    dclsb(i) = dc(squeeze(Lrc(:,1,i)), 1);

    fprintf(2, 'PRCL = %g CARM = %g \n', itf.PRCL, itf.CARM);
    fprintf(2, '     PRC car = %g ARM car = %g  PRC USB = %g PRC LSB = %g\n', dccar(i), abs(Cn(1,1,i))^2, ...
                dcusb(i), dclsb(i));
end
close(h);

% compute demodulated signals
[p,q] = ac(squeeze(Crc(:,1,:)), squeeze(Urc(:,1,:)), squeeze(Lrc(:,1,:)), itf.m, 0);

el = toc;
disp(['Elapsed time: ' num2str(el) ' s'])     

%%%%%% Plots

figure()
plot(pabs, abs(squeeze(Crc(1,1,:)).^2), 'r--')
hold all
plot(pabs, abs(squeeze(Urc(1,1,:)).^2), 'g--')
plot(pabs, abs(squeeze(Lrc(1,1,:)).^2), 'b--')
plot(pabs, dccar, 'r')
plot(pabs, dcusb, 'g')
plot(pabs, dclsb, 'b')

xlabel('Input mirror absorbed power [W]', 'fontsize', 15)
ylabel('PRC recycling gains', 'fontsize', 15)
grid
legend('CAR','USB','LSB')
title('PRCL tuned to maximize USB - CARM retuned to maximize carrier in arm', 'fontsize', 15)


figure()
plot(pabs, dccar0, 'r')
hold all
plot(pabs, dcusb0, 'g')
plot(pabs, dclsb0, 'b')
plot(pabs, abs(squeeze(Crc0(1,1,:)).^2), 'r--')
plot(pabs, abs(squeeze(Urc0(1,1,:)).^2), 'g--')
plot(pabs, abs(squeeze(Lrc0(1,1,:)).^2), 'b--')
xlabel('Input mirror absorbed power [W]', 'fontsize', 15)
ylabel('PRC recycling gains', 'fontsize', 15)
grid
legend('CAR (all modes)','USB (all modes)','LSB (all modes)', 'CAR (TEM00)','USB (TEM00)','LSB (TEM00)')
title('NDRC Bi-concave (not locked)', 'fontsize', 15)
set(gca, 'FontSize', 15)

figure()
plot(pabs, dccar, 'r')
hold all
plot(pabs, dcusb, 'g')
plot(pabs, dclsb, 'b')
plot(pabs, abs(squeeze(Crc(1,1,:)).^2), 'r--')
plot(pabs, abs(squeeze(Urc(1,1,:)).^2), 'g--')
plot(pabs, abs(squeeze(Lrc(1,1,:)).^2), 'b--')
xlabel('Input mirror absorbed power [W]', 'fontsize', 15)
ylabel('PRC recycling gains', 'fontsize', 15)
grid
legend('CAR (all modes)','USB (all modes)','LSB (all modes)', 'CAR (TEM00)','USB (TEM00)','LSB (TEM00)')
title('NDRC Bi-concave (locked)', 'fontsize', 15)
set(gca, 'FontSize', 15)

figure()
plot(pabs, dccar0, 'r--')
hold all
plot(pabs, dcusb0, 'g--')
plot(pabs, dclsb0, 'b--')
plot(pabs, dccar, 'r')
plot(pabs, dcusb, 'g')
plot(pabs, dclsb, 'b')
xlabel('Input mirror absorbed power [W]', 'fontsize', 15)
ylabel('PRC recycling gains', 'fontsize', 15)
grid
legend('CAR not locked','USB not locked','LSB not locked', 'CAR locked','USB locked','LSB locked')
set(gca, 'FontSize', 15)
title('NDRC Bi-concave', 'fontsize', 15)

figure()
plot(pabs, abs(squeeze(Cn(1,1,:))).^2)
hold all
plot(pabs, 10*abs(squeeze(Cn(1,4,:))).^2)
xlabel('Input mirror absorbed power [W]', 'fontsize', 15)
ylabel('Power inside arm cavity [W]', 'fontsize', 15)
legend('Carrier TEM 00', 'Carrier TEM 04 (x10)')
set(gca, 'fontsize', 15)
title('Power inside arm cavity', 'fontsize', 15)
grid

