% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Return the static fields in a dual-recycled interferometer for a unity
% input at frequency f
%
% [psi_rc, psi_sc, psi_df, psi_n, psi_w, psi_ref] = dualrecycled_fields(itf, f)
%
% itf = interferometer structure
% f   = field frequency
%
% psi_rc  = field inside power recycling (at PR level, going away from PR)
% psi_sc  = field inside signal recycling (at SR level, going toward SR)
% psi_df  = field at dark port
% psi_n   = field impinging on NE mirror, inside arm
% psi_w   = field impinging on WE mirror, inside arm
% psi_ref = field at reflection port
%


function [psi_rc, psi_sc, psi_df, psi_n, psi_w, psi_ref] = dualrecycled_fields(itf, f)
    
    k = 2*pi / 1.064e-6;
    c = 299792458;
    w = 2*pi*f;
    
    % compute propagators in recycling cavities
    modes = itf.ncav.modes;
    itf.PRC = diag(sqrt(-1j) * exp(1j * k * itf.PRCL - 1j * w/c * itf.Lprc) ...
        * exp(1j * sum(modes') * itf.phig_p));
    itf.PSC = diag(sqrt(-1j) * exp(1j*itf.phi_src/2 - 1j * w/c * itf.Lsrc) ...
        * exp(1j * sum(modes') * itf.phig_s));
    itf.PLm = sqrt(1j) * exp(1j * k * itf.MICH - 1j * w / c * itf.Schnupp/2);
    
    % compute fields
    N = length(itf.ncav.modes);
    U = eye(N);
    
    % Michelson operators
    itf.MSS = MSS(itf, f);
    itf.MAA = MAA(itf, f);
    itf.MSA = MSA(itf, f);
    itf.MAS = MAS(itf, f);
    
    % main fields
    SRC = inv(U - 1j*itf.PSC*itf.MAA*itf.PSC*itf.Rs);
    psi_rc = inv(U - 1j*itf.Rp*itf.PRC*itf.MSS*itf.PRC + ...
                itf.Rp*itf.PRC*itf.MAS*itf.PSC*itf.Rs* ...
                SRC * itf.PSC*itf.MSA*itf.PRC)*itf.Tp;
    psi_sc = SRC * itf.PSC*itf.MSA*itf.PRC * psi_rc;
    psi_df = itf.Ts * psi_sc;
    
    % Fabry-Perot cavity fields
    PLN = itf.ncav.PL * exp(1j*k*0.5 * (itf.CARM + itf.DARM)) * exp(-1j * w/c * itf.ncav.L);
    PLW = itf.wcav.PL * exp(1j*k*0.5 * (itf.CARM - itf.DARM)) * exp(-1j * w/c * itf.wcav.L);
    
    psi_n = 1/sqrt(2) * PLN * inv(U + itf.ncav.Ri*PLN*itf.ncav.Re*PLN) * ...
            itf.ncav.Ti*itf.ncav.Lens * itf.PLm * ...
            (itf.PRC - itf.PSC*itf.Rs * SRC * itf.PSC*itf.MSA*itf.PRC) * psi_rc;
    psi_w = 1j/sqrt(2) * PLW * inv(U + itf.wcav.Ri*PLW*itf.wcav.Re*PLW) * ...
            itf.wcav.Ti*itf.wcav.Lens * inv(itf.PLm) * ...
            (itf.PRC + itf.PSC*itf.Rs * SRC * itf.PSC*itf.MSA*itf.PRC) * psi_rc;
    
    psi_ref = (1j * itf.Rp' + 1j * itf.Tp * itf.Rp' * itf.Tp) - 1j * itf.Tp * itf.Rp' * psi_rc;
    
    
    
