% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Return the transmission matrix for a lens using a given mode basis
% 
% M = lens(f, w, modes, rmir, npt, thr)
%
%     f     = lens focal
%     w     = beam size at lens
%     modes = list of all modes to be used in [n,m] format
%     rmir  = mirror radius
%     npt   = number of points for numerical integration
%     thr   = below this values coupling coefficients are set to zero

function M = lens(f, w, modes, rlens, npt, thr)
    %tic;
    k = 2*pi / 1.064e-6;

    % build integration grid
    [x,y] = meshgrid(linspace(-rlens,rlens,npt),linspace(-rlens,rlens,npt));
    dxdy = (2*rlens / (npt - 1))^2;
    
    % build phase map function (using variables
    r2 = x.^2 + y.^2;
    Z = r2 / (4*f);
    
    % build Gaussian weight
    g = exp(-2*r2/w^2);
    
    % build mask for mirror radius
    msk = r2 < (rlens*rlens); 
    
    % compute matrix for each mode
    N = length(modes);
    M = zeros(N,N);
    %h = waitbar(0, 'Computing');
    for i=1:N
        for j=1:N
            m = modes(i,1);
            n = modes(i,2);
            p = modes(j,1);
            q = modes(j,2);
        
            norm = 1/sqrt(2^(n+m+p+q) * factorial(n) * factorial(m) * ...
                    factorial(p) * factorial(q));
            norm = norm * 2 / (pi * w^2) * dxdy;
                        
            % compute hermite polynomials (in correct variables)
            hm = polyval(HermitePoly(m), sqrt(2) * x / w);
            hn = polyval(HermitePoly(n), sqrt(2) * y / w);
            hp = polyval(HermitePoly(p), sqrt(2) * x / w);
            hq = polyval(HermitePoly(q), sqrt(2) * y / w);
            
            % integrate
            M(i,j) = norm * sum(sum(msk.* hp .* hq .* hm .* hn .* exp(2j*k*Z) .* g));
            
            % kill small elements
            if abs(M(i,j)) < thr
                M(i,j) = 0;
            end
            
           % el = toc;
           % ix = (i-1)*N + j;
           % eta = floor((el / ix * (N*N - ix)));
           % waitbar(ix/(N*N), h, ['Dual recycled locked (', num2str(floor(100*ix/(N*N))), '% ETA = ', num2str(eta), 's)']);
        end
    end
    %close(h);        
    %el = toc;
    %disp(['Elapsed time: ' num2str(el) ' s'])     
