% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Compute operator for astigmatic power recycling cavity. All astigmatism and
% Gouy phases are coded inside the recycling mirror operator (reflection
% and transmission).
%
% Recycling mirror are assumed to be perfectly matched, but can be tilted.
%
% function [R,T, RC, w] = ndrc_power_recycling(rec, cav, modes, theta1, theta2, theta3)
%
% rec   = structure for describing recyling cavity
%         rec.L1, rec.L2, rec.L3, rec.R2, rec.R3, rec.alpha
% cav   = structure describing arm cavity
% modes = list of modes to be used
% theta1, theta2, theta3 = tilts of PRM1, PRM2, PRM3
%
% R  = recycling mirror operator (reflection from BS to BS
% T  = recycling mirror operator (transmission from input to BS)
% RC = PRM radius of curvature (spherical and optimally matched)
% w  = beam size on PRM1



function [R, T, R1, w] = ndrc_signal_recycling(rec, cav, modes, theta1, theta2, theta3)
    global  npt thr 
    
    M = length(modes);
    
    %%%%%%%%%%%% COMPUTE SRM1 PARAMETERS
        
    % beam parameter at cavity reflection
    q0 = cav.L1 + 1j * cav.z0;
    
    % take into account (diverging) lens due to input mirror substrate
    n = 1.44963;
    MLI = [1, 0; (n-1)/cav.R1 1];
    
    % compute mode at input mirror exit
    q1 = ABCD(MLI, q0);
    
    % propagate gaussian parameter to SRM1 and compute Gouy phase
    % in the non-astigmatic basis
    ML1 = [1 rec.L1; 0 1];
    ML2 = [1 rec.L2; 0 1];
    ML3 = [1 rec.L3; 0 1];
    MR3 = [1 0; -2/rec.R3 1];
    MR2 = [1 0; -2/rec.R2 1];
    
    qsrm3in_0  = ABCD(ML3, q1);
    qsrm3out_0 = ABCD(MR3, qsrm3in_0);
    qsrm2in_0  = ABCD(ML2, qsrm3out_0);
    qsrm2out_0 = ABCD(MR2, qsrm2in_0);
    qsrm1in_0  = ABCD(ML1, qsrm2out_0);
    
    % compute beam parameters at SRM1
    R1 = 1/real(1/qsrm1in_0);
    w  = sqrt( -1.064e-6/(pi*imag(1/qsrm1in_0)));
    
    %%%%%%%%%%%% PROPAGATE CAVITY BEAM BASIS INSIDE SRC TOWARD SRM1
           
    % propagate gaussian parameter to SRM1 and compute Gouy phases
    % in the astigmatic basis
    a = rec.alpha / 180 * pi;
    
    MR3t = [1 0; -2/(rec.R3*cos(a)) 1];
    MR3s = [1 0; -2*cos(a)/rec.R3   1];
    MR2t = [1 0; -2/(rec.R2*cos(a)) 1];
    MR2s = [1 0; -2*cos(a)/rec.R2   1];
     
    qsrm3in_t  = ABCD(ML3, q1);
    qsrm3out_t = ABCD(MR3t, qsrm3in_t);
    qsrm2in_t  = ABCD(ML2, qsrm3out_t);
    qsrm2out_t = ABCD(MR2t, qsrm2in_t);
    qsrm1in_t  = ABCD(ML1, qsrm2out_t);
    phig_t_i3 = atan(real(qsrm3in_t)/imag(qsrm3in_t)) - atan(real(q1)/imag(q1));
    phig_t_23 = atan(real(qsrm2in_t)/imag(qsrm2in_t)) - atan(real(qsrm3out_t)/imag(qsrm3out_t));
    phig_t_12 = atan(real(qsrm1in_t)/imag(qsrm1in_t)) - atan(real(qsrm2out_t)/imag(qsrm2out_t));
         
    qsrm3in_s  = ABCD(ML3, q1);
    qsrm3out_s = ABCD(MR3s, qsrm3in_s);
    qsrm2in_s  = ABCD(ML2, qsrm3out_s);
    qsrm2out_s = ABCD(MR2s, qsrm2in_s);
    qsrm1in_s  = ABCD(ML1, qsrm2out_s);
    phig_s_i3 = atan(real(qsrm3in_s)/imag(qsrm3in_s)) - atan(real(q1)/imag(q1));
    phig_s_23 = atan(real(qsrm2in_s)/imag(qsrm2in_s)) - atan(real(qsrm3out_s)/imag(qsrm3out_s));
    phig_s_12 = atan(real(qsrm1in_s)/imag(qsrm1in_s)) - atan(real(qsrm2out_s)/imag(qsrm2out_s));
             
    %%%%%%%%%%%% REFLECT BEAM ON SRM1 AND BACK TO INPUT MIRROR
    
    MR1  = [1 0; -2/R1 1];
      
    Qsrm1out_t = ABCD(MR1, qsrm1in_t);   % backward modes are capital Q
    Qsrm2in_t  = ABCD(ML1, Qsrm1out_t);
    Qsrm2out_t = ABCD(MR2t, Qsrm2in_t);
    Qsrm3in_t  = ABCD(ML2, Qsrm2out_t);
    Qsrm3out_t = ABCD(MR3t, Qsrm3in_t);
    Q1_t       = ABCD(ML3, Qsrm3out_t);
    Phig_t_12 = atan(real(Qsrm2in_t)/imag(Qsrm2in_t)) - atan(real(Qsrm1out_t)/imag(Qsrm1out_t));
    Phig_t_23 = atan(real(Qsrm3in_t)/imag(Qsrm3in_t)) - atan(real(Qsrm2out_t)/imag(Qsrm2out_t));
    Phig_t_i3 = atan(real(Q1_t)/imag(Q1_t)) - atan(real(Qsrm3out_t)/imag(Qsrm3out_t));
    
    Qsrm1out_s = ABCD(MR1, qsrm1in_s);   % backward modes are capital Q
    Qsrm2in_s  = ABCD(ML1, Qsrm1out_s);
    Qsrm2out_s = ABCD(MR2s, Qsrm2in_s);
    Qsrm3in_s  = ABCD(ML2, Qsrm2out_s);
    Qsrm3out_s = ABCD(MR3s, Qsrm3in_s);
    Q1_s       = ABCD(ML3, Qsrm3out_s);
    Phig_s_12 = atan(real(Qsrm2in_s)/imag(Qsrm2in_s)) - atan(real(Qsrm1out_s)/imag(Qsrm1out_s));
    Phig_s_23 = atan(real(Qsrm3in_s)/imag(Qsrm3in_s)) - atan(real(Qsrm2out_s)/imag(Qsrm2out_s));
    Phig_s_i3 = atan(real(Q1_s)/imag(Q1_s)) - atan(real(Qsrm3out_s)/imag(Qsrm3out_s));
     
    
    %%%%%%%%%%%% COMPUTE ALL OPERATORS FROM INPUT MIRROR BACK TO INPUT MIRROR
    
    
    % propagation from input mirror to SRM3
    Pi3 = diag(1j * exp(1j * (modes(:,1) * phig_s_i3 + modes(:,2) * phig_t_i3)));
    % reflection on SRM3
    Msrm3 = mirror_reflection_qxy(rec.R3, rec.alpha, theta3, qsrm3in_t, qsrm3in_s, qsrm3out_t, qsrm3out_s, ...
        modes, 5, npt, thr);
    % propagation from SRM3 to SRM2
    P32 = diag(1j * exp(1j * (modes(:,1) * phig_s_23 + modes(:,2) * phig_t_23)));
    % reflection on SRM2
    Msrm2 = mirror_reflection_qxy(rec.R2, rec.alpha, theta2, qsrm2in_t, qsrm2in_s, qsrm2out_t, qsrm2out_s, ...
        modes, 5, npt, thr);
    % propagation from SRM2 to SRM1
    P21 = diag(1j * exp(1j * (modes(:,1) * phig_s_12 + modes(:,2) * phig_t_12)));
    % reflection on SRM1
    Msrm1 = mirror_reflection_qxy(R1, 0, theta1, qsrm1in_t, qsrm1in_s, Qsrm1out_t, Qsrm1out_s, ...
        modes, 5, npt, thr);
    % propagation from SRM1 to SRM2 
    P12 = diag(1j * exp(1j * (modes(:,1) * Phig_s_12 + modes(:,2) * Phig_t_12)));
    % reflection on SRM2
    Msrm2b = mirror_reflection_qxy(rec.R2, rec.alpha, theta2, Qsrm2in_t, Qsrm2in_s, Qsrm2out_t, Qsrm2out_s, ...
        modes, 5, npt, thr);
    % propagation from SRM2 to SRM3
    P23 = diag(1j * exp(1j * (modes(:,1) * Phig_s_23 + modes(:,2) * Phig_t_23)));
    % reflection on SRM3
    Msrm3b = mirror_reflection_qxy(rec.R3, rec.alpha, theta3, Qsrm3in_t, Qsrm3in_s, Qsrm3out_t, Qsrm3out_s, ...
        modes, 5, npt, thr);
    % propagation from SRM3 to input
    P3i = diag(1j * exp(1j * (modes(:,1) * Phig_s_i3 + modes(:,2) * Phig_t_i3)));
    
    %%%%%%%%%%%% PROJECTION BACK TO ROUND CAVITY BASIS
    
    MC = coupling_matrix(Q1_s, Q1_t, -conj(q1), -conj(q1), modes, 5, npt, thr);
    
    
    %%%%%%%%%%%% PUT EVERYTHING TOGHETER 
    %%%%%%%%%%%% (REFLECTION OPERATOR AS SEEN INSIDE ITF)
    
    % reflection from input to input
    R = MC * P3i * Msrm3b * P23 * Msrm2b * P12 * Msrm1 * P21 * Msrm2 * P32 * Msrm3 * Pi3;
    % rescale to maintain 00 resonant
    R = R / exp(1j * angle(R(1,1)));
  
    
    %%%%%%%%%%%% PUT EVERYTHING TOGHETER 
    %%%%%%%%%%%% (TRANSMISSION OPERATOR FROM INPUT MIRRORS TO OUTPUT BEAM)
    
    % projection of modes on SRM1 to round basis
    MC2 = coupling_matrix(qsrm1in_s, qsrm1in_t, qsrm1in_0, qsrm1in_0, modes, 5, npt, thr);
        
    T = MC2 * P21 * Msrm2 * P32 * Msrm3 * Pi3;
    
    
    
    
